/* bxCore.cpp */

#include "bxCore.h"
#include "CoreSub/bxCoreSub.h"
#include "CoreSub/bxErrChk.h"
#include "OrgTo3D/bxOrgTo3D.h"
#include "Element/bxElement.h"
#include "../Math/bxMath.h"
#include "Rect/bxRect.h"
#include "Tri/bxTri.h"
#include "Vector/bxVector.h"



bool  CbxCore::
GetDefaultAxis( const AxisForDefaultIn* in, AxisForDefaultOut* out )
{
	bool    result;
	double  normal[ 3 ];

	result = CbxCoreSub::NormalFromAxisForDefault( in, normal );
	if( result == true )
		return( true );

	result = CbxCoreSub::GetDefaultAxisMain( normal, out );
	if( result == true )
		return( true );

	return( false );
}


bool  CbxCore::
GetDefaultHandle( const HandleForTransIn* in, HandleForTransOut* out )
{
	bool    result;
	double  normal[ 3 ], radialVec[ 3 ], tanE[ 3 ], tan[ 3 ];

	CbxMath::Subtract3( in->radialVertexPos, in->vertexPos, radialVec );
	
	result = CbxCoreSub::NormalFromHandleForDefault( in, normal );
	if( result == true )
		return( true );

	CbxCoreSub::TanEfromHandleForDefault( radialVec, normal, tanE );
	if( CbxMath::Length3( tanE ) < CbxMath::nearZero )
		return( true );
	result = CbxVector::GetDefaultHandle( radialVec, tanE, tan );
	if( result == true )
		return( true );

	result = CbxCoreSub::GetDefaultHandleOut( in, radialVec, tan, out );
	if( result == true )
		return( true );

	return ( false );
}


bool  CbxCore::
GetDefaultCtlVec( const CtlVecForDefaultIn* in, bool* dispChk, CtlVecForDefaultOut* out )
{
	bool  result;
	CbxErrChk::ChkOnEdgeSurface  data;

	CbxErrChk::ConvCtlVecForDefaultToChkOnEdgeSurface( in, &data );
	result = CbxErrChk::DispChkOnEdgeSurface( &data, dispChk );
	if( result == true )
		return( true );
	else if( *dispChk == false )
		return( false );

	switch( in->numVertex ) {
		case 3:
			result = CbxTri::GetDefaultCtlVec( in->vertex, out->vertex );
			if( result == true )
				return( true );
			break;
		case 4:
			result = CbxRect::GetDefaultCtlVec( in->vertex, out->vertex );
			if( result == true )
				return( true );
			break;
		default:
			return( true );
	}

	return( false );
}



void  CbxCore::
SetVertexPos( const double newPos[ 3 ], double vertexPos[ 3 ] )
{
	CbxMath::Copy3( newPos, vertexPos );
}


bool  CbxCore::
RotVertex( const double origin[ 3 ], const double axis[ 3 ], const double deg, VertexForRotOut* rot )
{
	double  pos[ 3 ], mat[ 4 ][ 4 ];
	bool    errResult;

	CbxMath::RotateAxis4x4( deg, axis, mat, &errResult );
	if( errResult == true )
		return( true );

	CbxMath::Subtract3( rot->vertexPos, origin, pos );
	CbxMath::Transform3( pos, mat, pos );
	CbxMath::Add3( pos, origin, rot->vertexPos );

	CbxMath::Transform3( rot->axisVec[ 0 ], mat, rot->axisVec[ 0 ] );
	CbxMath::Transform3( rot->axisVec[ 1 ], mat, rot->axisVec[ 1 ] );

	return( false );
}


bool  CbxCore::
ScaleVertex( const double origin[ 3 ], const double scale, double vertexPos[ 3 ] )
{
	double  pos[ 3 ];

	if( scale < CbxMath::nearZero )
		return( true );

	CbxMath::Subtract3( vertexPos, origin, pos );
	CbxMath::Scale3( pos, scale, pos );
	CbxMath::Add3( pos, origin, vertexPos );

	return( false );
}


void  CbxCore::
SetHandleVec( const double newHandle[ 2 ], double handleVec[ 2 ] )
{
	CbxMath::Copy2( newHandle, handleVec );
	CbxErrChk::CorrectHandleVec1( &handleVec[ 1 ] );
}


bool  CbxCore::
SetHandleInfo( const double newHandleVec[ 3 ], const HandleForTransPlusIn* in, bool* dispChk, HandleForTransPlusOut* out )
{
	bool  result;
	CbxErrChk::ChkOnEdgeVertex  chkData;

	CbxErrChk::ConvHandleForTransPlusInToChkOnEdgeVertex( in, &chkData );
	CbxErrChk::DispChkOnEdgeVertex( &chkData, dispChk );
	if( *dispChk == false )
		return( false );

	result = CbxCoreSub::SetHandleInfo( newHandleVec, in, out );
	if( result == true )
		return( true );

	CbxErrChk::CorrectHandleVec1( &out->handleVec[ 1 ] );

	return( false );
}


void  CbxCore::
SetCtlPlaneDeg( const double newCtl, double* ctlPlaneDeg )
{
	*ctlPlaneDeg = newCtl;
	CbxErrChk::CorrectCtlPlaneDeg( ctlPlaneDeg );
}


void  CbxCore::
SetCtlVec( const double newCtl[ 2 ][ 2 ], double ctlVec[ 2 ][ 2 ] )
{
	short  i;

	for( i=0; i<2; i++ ) {
		CbxMath::Copy2( newCtl[ i ], ctlVec[ i ] );
		CbxErrChk::CorrectCtlVec( i, ctlVec[ i ] );
	}
}


bool  CbxCore::
SetCtlInfo( const double newCtlVec[ 3 ], const CtlVecForTransPlusIn* in, bool* dispChk, CtlVecForTransPlusOut* out )
{
	bool  result;
	CbxErrChk::ChkOnEdgeWing  chkData;

	CbxErrChk::ConvCtlVecForTransSubToChkOnEdgeWing( in->sub, &chkData );
	result = CbxErrChk::DispChkOnEdgeWing( &chkData, dispChk );
	if( result == true )
		return( true );
	else if( *dispChk == false )
		return( false );

	result = CbxCoreSub::SetCtlInfo( newCtlVec, in, out );
	if( result == true )
		return( true );

	CbxErrChk::CorrectCtlPlaneDeg( &out->ctlPlaneDeg );
	CbxErrChk::CorrectCtlVec( in->minusPlusNo, out->ctlVec );

	return( false );
}


void  CbxCore::
GetNormalOnVertex( const VertexForRot* data, double normalVec[ 3 ] )
{
	CbxMath::Cross3( data->out.axisVec[ 0 ], data->out.axisVec[ 1 ], normalVec );
	CbxMath::Normalize3( normalVec, CbxMath::zero3, normalVec );
}


bool  CbxCore::
GetHandle( const HandleForTrans* handle, bool* dispChk, double handleVec[ 3 ] )
{
	bool  result;
	CbxErrChk::ChkOnEdgeVertex  chkData;

	CbxErrChk::ConvHandleForTransToChkOnEdgeVertex( handle, &chkData );
	CbxErrChk::DispChkOnEdgeVertex( &chkData, dispChk );
	if( *dispChk == false )
		return( false );

	result = CbxOrgTo3Dex::GetHandle( handle, handleVec );
	if( result == true )
		return( true );

	return( false );
}


bool  CbxCore::
GetAxisInCtlPlaneDeg( const CtlPlaneDegForRot* ctl, bool* dispChk, double axisVec[ 3 ] )
{
	bool    result;
	short   i;
	double  len, handleVec[ 2 ][ 3 ];
	CbxErrChk::ChkOnEdgeRadial  chkData;

	CbxErrChk::ConvCtlPlaneDegForRotToChkOnEdgeRadial( ctl, &chkData );
	result = CbxErrChk::DispChkOnEdgeRadial( &chkData, dispChk );
	if( result == true )
		return( true );
	else if( *dispChk == false )
		return( false );

	len = CbxMath::Distance3( ctl->in.sub[ 0 ].vertexPos, ctl->in.sub[ 1 ].vertexPos );

	for( i=0; i<2; i++ ) {
		result = CbxOrgTo3D::HandleVec( ctl->in.sub[ i ].axisVec, len, ctl->in.sub[ i ].handleVec, handleVec[ i ] );
		if( result == true )
			return( true );
	}

	CbxElement::AxisInCtlPlaneDeg( ctl->in.sub[ 0 ].vertexPos, handleVec[ 0 ], handleVec[ 1 ],
		ctl->in.sub[ 1 ].vertexPos, axisVec );

	return( false );
}


bool  CbxCore::
GetNormalInCtlPlaneDeg( const CtlPlaneDegForRot* ctl, bool* dispChk, double normalVec[ 3 ] )
{
	bool  result;
	CbxErrChk::ChkOnEdgeRadial  chkData;

	CbxErrChk::ConvCtlPlaneDegForRotToChkOnEdgeRadial( ctl, &chkData );
	result = CbxErrChk::DispChkOnEdgeRadial( &chkData, dispChk );
	if( result == true )
		return( true );
	else if( *dispChk == false )
		return( false );

	result = CbxCoreSub::NormalInCtlPlaneDeg( ctl->in.sub, ctl->out.ctlPlaneDeg, normalVec );
	if( result == true )
		return( true );

	return( false );
}


bool  CbxCore::
GetAxisInCtlVec( const CtlVecForTrans* ctl,  bool* dispChk, double axis[ 2 ][ 3 ] )
{
	bool    result;
	double  lenBaseCtlVec;
	CbxErrChk::ChkOnEdgeWing  chkData;

	CbxErrChk::ConvCtlVecForTransSubToChkOnEdgeWing( ctl->in.sub, &chkData );
	result = CbxErrChk::DispChkOnEdgeWing( &chkData, dispChk );
	if( result == true )
		return( true );
	else if( *dispChk == false )
		return( false );

	result = CbxElementEx::ElementAxisInCtlVec( ctl, axis, &lenBaseCtlVec );
	if( result == true )
		return( true );

	return( false );
}


bool  CbxCore::
GetCtlInCtlVec( const CtlVecForTrans* ctl, bool* dispChk, double ctlVec[ 3 ] )
{
	bool  result;
	CbxErrChk::ChkOnEdgeWing  chkData;

	CbxErrChk::ConvCtlVecForTransSubToChkOnEdgeWing( ctl->in.sub, &chkData );
	result = CbxErrChk::DispChkOnEdgeWing( &chkData, dispChk );
	if( result == true )
		return( true );
	else if( *dispChk == false )
		return( false );

	result = CbxOrgTo3Dex::GetCtlInCtlVec( ctl, ctlVec );
	if( result == true )
		return( true );

	return( false );
}



bool  CbxCore::
ConvEdge( const HandleForTrans vertex[ 2 ], bool* dispChk, double bezierLine[ 4 ][ 3 ] )
{
	double  handleVec[ 2 ][ 3 ];
	short   i;
	bool    result;

	CbxMath::Copy3( vertex[ 0 ].in.vertexPos, bezierLine[ 0 ] );
	CbxMath::Copy3( vertex[ 1 ].in.vertexPos, bezierLine[ 3 ] );

	for( i=0; i<2; i++ ) {
		result = GetHandle( &vertex[ i ], dispChk, handleVec[ i ] );
		if( result == true )
			return( true );
		else if( *dispChk == false )
			return( false );
	}

	CbxMath::Add3( bezierLine[ 0 ], handleVec[ 0 ], bezierLine[ 1 ] );
	CbxMath::Add3( bezierLine[ 3 ], handleVec[ 1 ], bezierLine[ 2 ] );

	return( false );
}


bool  CbxCore::
ConvSurface( const SurfaceForPick* src, bool* dispChk, PatchRec* dst )
{
	bool  result;

	result = DispChkOnEdgeSurface( src, dispChk );
	if( result == true )
		return( true );
	else if( *dispChk == false )
		return( false );

	dst->numVertex = src->numVertex;
	switch( src->numVertex ) {
		case 3:
			result = CbxTri::ConvSurface( src->vertex, dst->tri );
			if( result == true )
				return( true );
			break;
		case 4:
			result = CbxRect::ConvSurface( src->vertex, &dst->rect );
			if( result == true )
				return( true );
			break;
		default:
			return( true );
	}

	return( false );
}


bool  CbxCore::
PosNormalSurface( const PatchRec* patch, const double uv[ 2 ], double pos[ 3 ], double normalVec[ 3 ] )
{
	switch( patch->numVertex ) {
		case 3:
			CbxTri::PosNormalSurface( patch->tri, uv, pos, normalVec );
			break;
		case 4:
			CbxRect::PosNormalSurface( &patch->rect, uv, pos, normalVec );
			break;
		default:
			return( true );
	}

	return( false );
}


bool  CbxCore::
PosSurface( const PatchRec* patch, const double uv[ 2 ], double pos[ 3 ] )
{
	switch( patch->numVertex ) {
		case 3:
			CbxTri::PosSurface( patch->tri, uv, pos );
			break;
		case 4:
			CbxRect::PosSurface( &patch->rect, uv, pos );
			break;
		default:
			return( true );
	}

	return( false );
}


bool  CbxCore::
NormalSurface( const PatchRec* patch, const double uv[ 2 ], double normalVec[ 3 ] )
{
	switch( patch->numVertex ) {
		case 3:
			CbxTri::NormalSurface( patch->tri, uv, normalVec );
			break;
		case 4:
			CbxRect::NormalSurface( &patch->rect, uv, normalVec );
			break;
		default:
			return( true );
	}

	return( false );
}



bool  CbxCore::
ErrChkAxisVec( const double axisVec[ 2 ][ 3 ] )
{
	return( CbxErrChk::ErrChkAxisVec( axisVec ) );
}


bool  CbxCore::
ErrChkHandleVec1( const double handleVec1 /* handleVec[ 1 ] */ )
{
	return( CbxErrChk::ErrChkHandleVec1( handleVec1 ) );
}


bool  CbxCore::
ErrChkCtlPlaneDeg( const double ctlPlaneDeg )
{
	return( CbxErrChk::ErrChkCtlPlaneDeg( ctlPlaneDeg ) );
}


bool  CbxCore::
ErrChkCtlVec( const double ctlVec[ 2 ][ 2 ] )
{
	return( CbxErrChk::ErrChkCtlVec( ctlVec ) );
}


ushort  CbxCore::
MaxNumRadial( void )
{
	return( CbxErrChk::MaxNumRadial() );
}


bool  CbxCore::
DispChkOnEdgeSurface( const SurfaceForPick* data, bool* dispChk )
{
	return( CbxCoreSub::DispChkOnEdgeSurface( data, dispChk ) );
}
